@@ -49,10 +49,7 @@
 
 package com.openexchange.imap;
 
-import java.lang.reflect.Field;
-import java.util.Arrays;
 import java.util.HashSet;
-import java.util.Hashtable;
 import java.util.Map;
 import java.util.Set;
 import java.util.concurrent.ConcurrentMap;
@@ -84,7 +81,6 @@ import com.openexchange.mail.usersetting.UserSettingMail;
 import com.openexchange.mail.usersetting.UserSettingMailStorage;
 import com.openexchange.session.Session;
 import com.sun.mail.imap.IMAPFolder;
-import com.sun.mail.imap.IMAPMessage;
 import com.sun.mail.imap.IMAPStore;
 import com.sun.mail.imap.Rights.Right;
 
@@ -156,7 +152,7 @@ public abstract class IMAPFolderWorker extends MailMessageStorageLong {
      * @param fullName The mailbox full name
      * @throws MessagingException If such a fail-fast error exists and has not elapsed, yet
      */
-    public static void checkFailFast(final IMAPStore imapStore, final String fullName) throws MessagingException {
+    public static void checkFailFast(IMAPStore imapStore, String fullName) throws MessagingException {
         if (null == imapStore || null == fullName) {
             return;
         }
@@ -171,8 +167,22 @@ public abstract class IMAPFolderWorker extends MailMessageStorageLong {
         FAIL_FAST.remove(key, failFastError);
     }
 
-    /*
-     * Fields
+    /**
+     * Marks specified folder for fail-fast error (if not already marked).
+     *
+     * @param imapStore The IMAP store
+     * @param fullName The mailbox full name
+     * @param e The associated error
+     */
+    public static void markForFailFast(IMAPStore imapStore, String fullName, MessagingException e) {
+        if (null == imapStore || null == fullName || null == e) {
+            return;
+        }
+        FAIL_FAST.putIfAbsent(new StringBuilder(fullName).append('@').append(imapStore.toString()).toString(), new FailFastError(e));
+    }
+
+    /*-
+     * ----------------------------------------------- Fields ---------------------------------------------------------------------
      */
 
     protected final IMAPStore imapStore;
@@ -382,6 +392,7 @@ public abstract class IMAPFolderWorker extends MailMessageStorageLong {
              */
             synchronized (imapFolder) {
                 IMAPCommandsCollection.forceNoopCommand(imapFolder);
+                clearCache(imapFolder);
                 try {
                     /*
                      * This call also checks if folder is opened
@@ -391,9 +402,6 @@ public abstract class IMAPFolderWorker extends MailMessageStorageLong {
                         /*
                          * Identical folder is already opened in an appropriate mode.
                          */
-                        // IMAPCommandsCollection.updateIMAPFolder(imapFolder,
-                        // mode);
-                        clearCache(imapFolder);
                         return imapFolder;
                     }
                     /*
@@ -459,47 +467,25 @@ public abstract class IMAPFolderWorker extends MailMessageStorageLong {
         synchronized (retval) {
             final ListLsubEntry listEntry = ListLsubCache.getCachedLISTEntry(fullName, accountId, retval, session);
             if (!isDefaultFolder && !STR_INBOX.equals(fullName) && (!listEntry.exists())) {
-                throw IMAPException.create(
-                    IMAPException.Code.FOLDER_NOT_FOUND,
-                    imapConfig,
-                    session,
-                    isDefaultFolder ? MailFolder.DEFAULT_FOLDER_NAME : fullName);
+                ListLsubCache.clearCache(accountId, session);
+                throw IMAPException.create(IMAPException.Code.FOLDER_NOT_FOUND, imapConfig, session, isDefaultFolder ? MailFolder.DEFAULT_FOLDER_NAME : fullName);
             }
             if ((desiredMode != Folder.READ_ONLY) && (desiredMode != Folder.READ_WRITE)) {
                 throw IMAPException.create(IMAPException.Code.UNKNOWN_FOLDER_MODE, imapConfig, session, Integer.valueOf(desiredMode));
             }
             final boolean selectable = listEntry.canOpen();
             if (!selectable) { // NoSelect
-                throw IMAPException.create(
-                    IMAPException.Code.FOLDER_DOES_NOT_HOLD_MESSAGES,
-                    imapConfig,
-                    session,
-                    isDefaultFolder ? MailFolder.DEFAULT_FOLDER_NAME : fullName);
+                throw IMAPException.create(IMAPException.Code.FOLDER_DOES_NOT_HOLD_MESSAGES, imapConfig, session, isDefaultFolder ? MailFolder.DEFAULT_FOLDER_NAME : fullName);
             }
             try {
                 if (imapConfig.isSupportsACLs() && !aclExtension.canRead(RightsCache.getCachedRights(retval, true, session, accountId))) {
-                    throw IMAPException.create(
-                        IMAPException.Code.NO_FOLDER_OPEN,
-                        imapConfig,
-                        session,
-                        isDefaultFolder ? MailFolder.DEFAULT_FOLDER_NAME : fullName);
+                    throw IMAPException.create(IMAPException.Code.NO_FOLDER_OPEN, imapConfig, session, isDefaultFolder ? MailFolder.DEFAULT_FOLDER_NAME : fullName);
                 }
             } catch (final MessagingException e) {
-                throw IMAPException.create(
-                    IMAPException.Code.NO_ACCESS,
-                    imapConfig,
-                    session,
-                    e,
-                    isDefaultFolder ? MailFolder.DEFAULT_FOLDER_NAME : fullName);
+                throw IMAPException.create(IMAPException.Code.NO_ACCESS, imapConfig, session, e, isDefaultFolder ? MailFolder.DEFAULT_FOLDER_NAME : fullName);
             }
-            if ((Folder.READ_WRITE == desiredMode) && (!selectable) && STR_FALSE.equalsIgnoreCase(imapAccess.getMailProperties().getProperty(
-                MimeSessionPropertyNames.PROP_ALLOWREADONLYSELECT,
-                STR_FALSE)) && IMAPCommandsCollection.isReadOnly(retval)) {
-                throw IMAPException.create(
-                    IMAPException.Code.READ_ONLY_FOLDER,
-                    imapConfig,
-                    session,
-                    isDefaultFolder ? MailFolder.DEFAULT_FOLDER_NAME : fullName);
+            if ((Folder.READ_WRITE == desiredMode) && (!selectable) && STR_FALSE.equalsIgnoreCase(imapAccess.getMailProperties().getProperty(MimeSessionPropertyNames.PROP_ALLOWREADONLYSELECT, STR_FALSE)) && IMAPCommandsCollection.isReadOnly(retval)) {
+                throw IMAPException.create(IMAPException.Code.READ_ONLY_FOLDER, imapConfig, session, isDefaultFolder ? MailFolder.DEFAULT_FOLDER_NAME : fullName);
             }
             openFolder(desiredMode, retval);
         }
@@ -520,46 +506,16 @@ public abstract class IMAPFolderWorker extends MailMessageStorageLong {
         return MailExceptionCode.UNEXPECTED_ERROR.create(e, e.getMessage());
     }
 
-    protected static volatile Field messagesField;
-    protected static volatile Field messageCacheField;
-    protected static volatile Field uidTableField;
-
-    /** Clears the cache */
-    protected static void clearCache(final IMAPFolder imapFolder) {
+    /**
+     * Clears given IMAP folder's {@link com.sun.mail.imap.MessageCache message cache} and UID table.
+     *
+     * @param imapFolder The IMAP folder
+     */
+    public static void clearCache(final IMAPFolder imapFolder) {
         if (null == imapFolder) {
             return;
         }
-        final Field messageCacheField = IMAPFolderWorker.messageCacheField;
-        if (null == messageCacheField) {
-            return;
-        }
-        final Field messagesField = IMAPFolderWorker.messagesField;
-        if (null == messagesField) {
-            return;
-        }
-        final Field uidTableField = IMAPFolderWorker.uidTableField;
-        if (null == uidTableField) {
-            return;
-        }
-
-        try {
-            final com.sun.mail.imap.MessageCache mc = (com.sun.mail.imap.MessageCache) messageCacheField.get(imapFolder);
-            if (null != mc) {
-                final IMAPMessage[] messages = (IMAPMessage[]) messagesField.get(mc);
-                if (null != messages) {
-                    Arrays.fill(messages, null);
-                }
-            }
-
-            final Hashtable<?, ?> uidTable = (Hashtable<?, ?>) uidTableField.get(imapFolder);
-            if (null != uidTable) {
-                uidTable.clear();
-            }
-        } catch (final IllegalArgumentException e) {
-            LOG.error("", e);
-        } catch (final IllegalAccessException e) {
-            LOG.error("", e);
-        }
+        imapFolder.clearMessageCache();
     }
 
     /** Safely clears the cache */
