@@ -49,10 +49,12 @@
 
 package com.openexchange.imap.cache;
 
+import static com.openexchange.java.Strings.isEmpty;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.List;
 import java.util.concurrent.Callable;
+import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.ConcurrentMap;
 import java.util.concurrent.ExecutionException;
 import java.util.concurrent.Future;
@@ -60,8 +62,12 @@ import java.util.concurrent.FutureTask;
 import javax.mail.MessagingException;
 import org.cliffc.high_scale_lib.NonBlockingHashMap;
 import org.slf4j.Logger;
+import com.openexchange.caching.CacheService;
+import com.openexchange.caching.events.CacheEvent;
+import com.openexchange.caching.events.CacheEventService;
 import com.openexchange.exception.OXException;
 import com.openexchange.imap.config.IMAPProperties;
+import com.openexchange.imap.services.Services;
 import com.openexchange.mail.MailExceptionCode;
 import com.openexchange.mail.mime.MimeMailException;
 import com.openexchange.session.Session;
@@ -80,12 +86,12 @@ public final class ListLsubCache {
      */
     protected static final Logger LOG = org.slf4j.LoggerFactory.getLogger(ListLsubCache.class);
 
+    private static final ListLsubCache INSTANCE = new ListLsubCache();
+
     private static final class Key {
 
         private final int cid;
-
         private final int user;
-
         private final int hash;
 
         protected Key(final int user, final int cid) {
@@ -124,15 +130,57 @@ public final class ListLsubCache {
 
     } // End of class Key
 
-    private static Key keyFor(final Session session) {
-        return new Key(session.getUserId(), session.getContextId());
-    }
+    private static final class KeyedCache {
+
+        private final Key key;
+
+        KeyedCache(Key key) {
+            super();
+            this.key = key;
+        }
+
+        /**
+         * Gets the associated object from cache
+         *
+         * @return The object or <code>null</code>
+         */
+        ConcurrentMap<Integer, Future<ListLsubCollection>> get() {
+            return CACHE.get(key);
+        }
+
+        /**
+         * Safely puts given object into cache
+         *
+         * @param value The object to put
+         * @return The object previously in cache or <code>null</code>
+         * @throws OXException If there is already such an entry in cache
+         */
+        ConcurrentMap<Integer, Future<ListLsubCollection>> putIfAbsent(ConcurrentMap<Integer, Future<ListLsubCollection>> map) {
+            return CACHE.putIfAbsent(key, map);
+        }
+
+        /**
+         * Removes the object from cache
+         */
+        void remove() {
+            CACHE.remove(key);
+        }
+
+    } // End of class KeyedCache
 
     /**
-     * The virtual region name.
+     * The region name.
      */
     public static final String REGION = "ListLsubCache";
 
+    private static KeyedCache getCache(Session session) {
+        return getCache(session.getUserId(), session.getContextId());
+    }
+
+    private static KeyedCache getCache(int userId, int contextId) {
+        return new KeyedCache(new Key(userId, contextId));
+    }
+
     /** The default timeout for LIST/LSUB cache (5 minutes) */
     private static final long DEFAULT_TIMEOUT = 300000;
 
@@ -142,7 +190,8 @@ public final class ListLsubCache {
 
     private static final boolean DO_GETACL = false;
 
-    private static final ConcurrentMap<Key, ConcurrentMap<Integer, Future<ListLsubCollection>>> MAP = new NonBlockingHashMap<Key, ConcurrentMap<Integer, Future<ListLsubCollection>>>();
+    /** The cache */
+    static final ConcurrentMap<Key, ConcurrentMap<Integer, Future<ListLsubCollection>>> CACHE = new NonBlockingHashMap<Key, ConcurrentMap<Integer, Future<ListLsubCollection>>>();
 
     /**
      * No instance
@@ -156,7 +205,7 @@ public final class ListLsubCache {
      *
      * @param session The session providing user information
      */
-    public static void dropFor(final Session session) {
+    public static void dropFor(Session session) {
         if (null != session) {
             dropFor(session.getUserId(), session.getContextId());
         }
@@ -168,9 +217,23 @@ public final class ListLsubCache {
      * @param userId The user identifier
      * @param contextId The context identifier
      */
-    public static void dropFor(final int userId, final int contextId) {
-        MAP.remove(new Key(userId, contextId));
-        LOG.debug("Cleaned user-sensitive LIST/LSUB cache for user {} in context {}", userId, contextId);
+    public static void dropFor(int userId, int contextId) {
+        dropFor(userId, contextId, true);
+    }
+
+    /**
+     * Drop caches for given user.
+     *
+     * @param userId The user identifier
+     * @param contextId The context identifier
+     * @param notify Whether to notify
+     */
+    public static void dropFor(int userId, int contextId, boolean notify) {
+        getCache(userId, contextId).remove();
+        if (notify) {
+            fireInvalidateCacheEvent(userId, contextId);
+        }
+        LOG.debug("Cleaned user-sensitive LIST/LSUB cache for user {} in context {}", Integer.valueOf(userId), Integer.valueOf(contextId));
     }
 
     /**
@@ -181,15 +244,14 @@ public final class ListLsubCache {
      * @param session The session
      */
     public static void removeCachedEntry(final String fullName, final int accountId, final Session session) {
-        final ConcurrentMap<Integer, Future<ListLsubCollection>> map = MAP.get(keyFor(session));
-        if (null == map) {
-            return;
-        }
-        final ListLsubCollection collection = getSafeFrom(map.get(Integer.valueOf(accountId)));
+        ConcurrentMap<Integer, Future<ListLsubCollection>> map = getCache(session).get();
+        ListLsubCollection collection = getSafeFrom(map.get(Integer.valueOf(accountId)));
         if (null != collection) {
             synchronized (collection) {
                 collection.remove(fullName);
             }
+
+            fireInvalidateCacheEvent(session);
         }
     }
 
@@ -229,15 +291,14 @@ public final class ListLsubCache {
      * @param contextId The context identifier
      */
     public static void clearCache(final int accountId, final int userId, final int contextId) {
-        final ConcurrentMap<Integer, Future<ListLsubCollection>> map = MAP.get(new Key(userId, contextId));
-        if (null == map) {
-            return;
-        }
-        final ListLsubCollection collection = getSafeFrom(map.get(Integer.valueOf(accountId)));
+        ConcurrentMap<Integer, Future<ListLsubCollection>> map = getCache(userId, contextId).get();
+        ListLsubCollection collection = getSafeFrom(map.get(Integer.valueOf(accountId)));
         if (null != collection) {
             synchronized (collection) {
                 collection.clear();
             }
+
+            fireInvalidateCacheEvent(userId, contextId);
         }
     }
 
@@ -258,6 +319,8 @@ public final class ListLsubCache {
                 return;
             }
             collection.addSingle(fullName, imapFolder, DO_STATUS, DO_GETACL);
+
+            fireInvalidateCacheEvent(session);
         }
     }
 
@@ -331,6 +394,7 @@ public final class ListLsubCache {
              * Update & re-check
              */
             collection.update(fullName, imapFolder, DO_STATUS, DO_GETACL);
+            fireInvalidateCacheEvent(session);
             entry = collection.getLsub(fullName);
             return null == entry ? ListLsubCollection.emptyEntryFor(fullName) : entry;
         }
@@ -372,6 +436,7 @@ public final class ListLsubCache {
                  * Update & re-check
                  */
                 collection.update(fullName, imapFolder, DO_STATUS, DO_GETACL);
+                fireInvalidateCacheEvent(session);
                 entry = collection.getList(fullName);
                 return null == entry ? ListLsubCollection.emptyEntryFor(fullName) : entry;
             }
@@ -461,6 +526,7 @@ public final class ListLsubCache {
              * Update & re-check
              */
             collection.update(fullName, imapFolder, DO_STATUS, DO_GETACL);
+            fireInvalidateCacheEvent(session);
             entry = collection.getList(fullName);
             return null == entry ? ListLsubCollection.emptyEntryFor(fullName) : entry;
         }
@@ -497,43 +563,44 @@ public final class ListLsubCache {
      * @throws OXException If loading the entry fails
      * @throws MessagingException If a messaging error occurs
      */
-    public static List<ListLsubEntry> getAllEntries(final String optParentFullName, final int accountId, final boolean subscribedOnly, final IMAPStore imapStore, final Session session) throws OXException, MessagingException {
-        final IMAPFolder imapFolder = (IMAPFolder) imapStore.getDefaultFolder();
-        final ListLsubCollection collection = getCollection(accountId, imapFolder, session);
+    public static List<ListLsubEntry> getAllEntries(String optParentFullName, int accountId, boolean subscribedOnly, IMAPStore imapStore, Session session) throws OXException, MessagingException {
+        IMAPFolder imapFolder = (IMAPFolder) imapStore.getDefaultFolder();
+        ListLsubCollection collection = getCollection(accountId, imapFolder, session);
         if (isAccessible(collection)) {
-            if (null != optParentFullName) {
-                final ListLsubEntry entry = subscribedOnly ? collection.getLsub(optParentFullName) : collection.getList(optParentFullName);
-                if (null != entry) {
-                    return entry.getChildren();
-                }
-            } else {
+            if (null == optParentFullName) {
                 return subscribedOnly ? collection.getLsubs() : collection.getLists();
             }
+
+            ListLsubEntry entry = subscribedOnly ? collection.getLsub(optParentFullName) : collection.getList(optParentFullName);
+            if (null != entry) {
+                return entry.getChildren();
+            }
         }
         synchronized (collection) {
             if (checkTimeStamp(imapFolder, collection)) {
-                if (null != optParentFullName) {
-                    final ListLsubEntry entry = subscribedOnly ? collection.getLsub(optParentFullName) : collection.getList(optParentFullName);
-                    if (null != entry) {
-                        return entry.getChildren();
-                    }
-                } else {
+                if (null == optParentFullName) {
                     return subscribedOnly ? collection.getLsubs() : collection.getLists();
                 }
+
+                ListLsubEntry entry = subscribedOnly ? collection.getLsub(optParentFullName) : collection.getList(optParentFullName);
+                if (null != entry) {
+                    return entry.getChildren();
+                }
             }
             /*
              * Update & re-check
              */
             collection.reinit(imapStore, DO_STATUS, DO_GETACL);
-            if (null != optParentFullName) {
-                final ListLsubEntry entry = subscribedOnly ? collection.getLsub(optParentFullName) : collection.getList(optParentFullName);
-                if (null != entry) {
-                    return entry.getChildren();
-                }
-                return Collections.emptyList();
-            } else {
+            fireInvalidateCacheEvent(session);
+            if (null == optParentFullName) {
                 return subscribedOnly ? collection.getLsubs() : collection.getLists();
             }
+
+            ListLsubEntry entry = subscribedOnly ? collection.getLsub(optParentFullName) : collection.getList(optParentFullName);
+            if (null != entry) {
+                return entry.getChildren();
+            }
+            return Collections.emptyList();
         }
     }
 
@@ -548,7 +615,7 @@ public final class ListLsubCache {
      * @throws OXException If loading the entry fails
      * @throws MessagingException If a messaging error occurs
      */
-    public static ListLsubEntry[] getCachedEntries(final String fullName, final int accountId, final IMAPFolder imapFolder, final Session session) throws OXException, MessagingException {
+    public static ListLsubEntry[] getCachedEntries(String fullName, int accountId, IMAPFolder imapFolder, Session session) throws OXException, MessagingException {
         final ListLsubCollection collection = getCollection(accountId, imapFolder, session);
         if (isAccessible(collection)) {
             final ListLsubEntry listEntry = collection.getList(fullName);
@@ -574,6 +641,7 @@ public final class ListLsubCache {
                  * Update & re-check
                  */
                 collection.update(fullName, imapFolder, DO_STATUS, DO_GETACL);
+                fireInvalidateCacheEvent(session);
                 listEntry = collection.getList(fullName);
             }
             final ListLsubEntry lsubEntry = collection.getLsub(fullName);
@@ -695,32 +763,33 @@ public final class ListLsubCache {
     }
 
     private static ListLsubCollection getCollection(final int accountId, final IMAPFolder imapFolder, final Session session) throws OXException, MessagingException {
-        /*
-         * Initialize appropriate cache entry
-         */
-        final Key key = keyFor(session);
-        ConcurrentMap<Integer, Future<ListLsubCollection>> map = MAP.get(key);
+        KeyedCache cache = getCache(session);
+
+        // Get the associated map
+        ConcurrentMap<Integer, Future<ListLsubCollection>> map = cache.get();
         if (null == map) {
-            final ConcurrentMap<Integer, Future<ListLsubCollection>> newmap = new NonBlockingHashMap<Integer, Future<ListLsubCollection>>();
-            map = MAP.putIfAbsent(key, newmap);
+            ConcurrentHashMap<Integer, Future<ListLsubCollection>> newmap = new ConcurrentHashMap<Integer, Future<ListLsubCollection>>();
+            map = cache.putIfAbsent(newmap);
             if (null == map) {
                 map = newmap;
             }
         }
+
+        // Submit task
         Future<ListLsubCollection> f = map.get(Integer.valueOf(accountId));
         boolean caller = false;
         if (null == f) {
-            final FutureTask<ListLsubCollection> ft = new FutureTask<ListLsubCollection>(new Callable<ListLsubCollection>() {
+            FutureTask<ListLsubCollection> ft = new FutureTask<ListLsubCollection>(new Callable<ListLsubCollection>() {
 
                 @Override
                 public ListLsubCollection call() throws OXException, MessagingException {
                     String[] shared;
                     String[] user;
                     try {
-                        final IMAPStore imapStore = (IMAPStore) imapFolder.getStore();
+                        IMAPStore imapStore = (IMAPStore) imapFolder.getStore();
                         try {
                             shared = check(NamespaceFoldersCache.getSharedNamespaces(imapStore, true, session, accountId));
-                        } catch (final MessagingException e) {
+                        } catch (MessagingException e) {
                             if (imapStore.hasCapability("NAMESPACE")) {
                                 LOG.warn("Couldn't get shared namespaces.", e);
                             } else {
@@ -733,18 +802,18 @@ public final class ListLsubCache {
                         }
                         try {
                             user = check(NamespaceFoldersCache.getUserNamespaces(imapStore, true, session, accountId));
-                        } catch (final MessagingException e) {
+                        } catch (MessagingException e) {
                             if (imapStore.hasCapability("NAMESPACE")) {
                                 LOG.warn("Couldn't get user namespaces.", e);
                             } else {
                                 LOG.debug("Couldn't get user namespaces.", e);
                             }
                             user = new String[0];
-                        } catch (final RuntimeException e) {
+                        } catch (RuntimeException e) {
                             LOG.warn("Couldn't get user namespaces.", e);
                             user = new String[0];
                         }
-                    } catch (final MessagingException e) {
+                    } catch (MessagingException e) {
                         throw MimeMailException.handleMessagingException(e);
                     }
                     return new ListLsubCollection(imapFolder, shared, user, DO_STATUS, DO_GETACL);
@@ -759,21 +828,21 @@ public final class ListLsubCache {
         }
         try {
             return getFrom(f);
-        } catch (final OXException e) {
+        } catch (OXException e) {
             if (caller) {
-                MAP.remove(key);
+                cache.remove();
             }
             throw e;
-        } catch (final InterruptedException e) {
+        } catch (InterruptedException e) {
             Thread.currentThread().interrupt();
             throw MailExceptionCode.INTERRUPT_ERROR.create(e, e.getMessage());
         }
     }
 
-    static String[] check(final String[] array) {
-        final List<String> list = new ArrayList<String>(array.length);
+    static String[] check(String[] array) {
+        List<String> list = new ArrayList<String>(array.length);
         for (int i = 0; i < array.length; i++) {
-            final String s = array[i];
+            String s = array[i];
             if (!isEmpty(s)) {
                 list.add(s);
             }
@@ -781,18 +850,6 @@ public final class ListLsubCache {
         return list.toArray(new String[list.size()]);
     }
 
-    private static boolean isEmpty(final String string) {
-        if (null == string) {
-            return true;
-        }
-        final int len = string.length();
-        boolean isWhitespace = true;
-        for (int i = 0; isWhitespace && i < len; i++) {
-            isWhitespace = com.openexchange.java.Strings.isWhitespace(string.charAt(i));
-        }
-        return isWhitespace;
-    }
-
     /**
      * Checks for any subscribed subfolder.
      *
@@ -858,4 +915,30 @@ public final class ListLsubCache {
         }
     }
 
+    private static void fireInvalidateCacheEvent(Session session) {
+        fireInvalidateCacheEvent(session.getUserId(), session.getContextId());
+    }
+
+    private static void fireInvalidateCacheEvent(int userId, int contextId) {
+        CacheEventService cacheEventService = Services.optService(CacheEventService.class);
+        if (null != cacheEventService && cacheEventService.getConfiguration().remoteInvalidationForPersonalFolders()) {
+            CacheEvent event = newCacheEventFor(userId, contextId);
+            if (null != event) {
+                cacheEventService.notify(INSTANCE, event, false);
+            }
+        }
+    }
+
+    /**
+     * Creates a new cache event
+     *
+     * @param userId The user identifier
+     * @param contextId The context identifier
+     * @return The cache event
+     */
+    private static CacheEvent newCacheEventFor(int userId, int contextId) {
+        CacheService service = Services.optService(CacheService.class);
+        return null == service ? null : CacheEvent.INVALIDATE(REGION, Integer.toString(contextId), service.newCacheKey(contextId, userId));
+    }
+
 }
