@@ -552,6 +552,13 @@ public final class MailMessageFetchIMAPCommand extends AbstractIMAPCommand<MailM
                         mailMessage.addBcc(MimeMessageConverter.getAddressHeader(hdr.getValue()));
                     }
                 });
+                put(MessageHeaders.HDR_REPLY_TO, new HeaderHandler() {
+
+                    @Override
+                    public void handle(final Header hdr, final IDMailMessage mailMessage) throws OXException {
+                        mailMessage.addReplyTo(MimeMessageConverter.getAddressHeader(hdr.getValue()));
+                    }
+                });
                 put(MessageHeaders.HDR_DISP_NOT_TO, new HeaderHandler() {
 
                     @Override
@@ -834,21 +841,7 @@ public final class MailMessageFetchIMAPCommand extends AbstractIMAPCommand<MailM
             msg.addTo(wrap(env.to));
             msg.addCc(wrap(env.cc));
             msg.addBcc(wrap(env.bcc));
-            try {
-                final InternetAddress[] replyTo = QuotedInternetAddress.toQuotedAddresses(env.replyTo);
-                if (null != replyTo && replyTo.length > 0) {
-                    for (final InternetAddress internetAddress : replyTo) {
-                        msg.addHeader("Reply-To", internetAddress.toString());
-                    }
-                }
-            } catch (final AddressException addressException) {
-                final InternetAddress[] replyTo = env.replyTo;
-                if (null != replyTo && replyTo.length > 0) {
-                    for (final InternetAddress internetAddress : replyTo) {
-                        msg.addHeader("Reply-To", internetAddress.toString());
-                    }
-                }
-            }
+            msg.addReplyTo(wrap(env.replyTo));
             msg.setHeader("In-Reply-To", env.inReplyTo);
             msg.setHeader("Message-Id", env.messageId);
             msg.setSubject(MimeMessageUtility.decodeEnvelopeSubject(env.subject));
@@ -877,11 +870,8 @@ public final class MailMessageFetchIMAPCommand extends AbstractIMAPCommand<MailM
             msg.addTo((InternetAddress[]) message.getRecipients(RecipientType.TO));
             msg.addCc((InternetAddress[]) message.getRecipients(RecipientType.CC));
             msg.addBcc((InternetAddress[]) message.getRecipients(RecipientType.BCC));
-            String[] header = message.getHeader("Reply-To");
-            if (null != header && header.length > 0) {
-                msg.addHeader("Reply-To", header[0]);
-            }
-            header = message.getHeader("In-Reply-To");
+            msg.addReplyTo((InternetAddress[]) message.getReplyTo());
+            String[] header = message.getHeader("In-Reply-To");
             if (null != header && header.length > 0) {
                 msg.addHeader("In-Reply-To", header[0]);
             }
