@@ -188,6 +188,13 @@ public interface IIMAPProperties extends IMailProperties {
      */
     public boolean allowFolderCaches();
 
+    /**
+     * Checks whether it is allowed to FETCH single headers
+     *
+     * @return <code>true</code> if allowed; otherwise <code>false</code>
+     */
+    public boolean allowFetchSingleHeaders();
+
     /**
      * Gets supported SSL protocols
      *
