@@ -137,6 +137,8 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
 
     private boolean allowFolderCaches;
 
+    private boolean allowFetchSingleHeaders;
+
     private String sContainerType;
 
     private String sslProtocols;
@@ -154,6 +156,8 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
         newACLExtMap = new NonBlockingHashMap<String, Boolean>();
         mailProperties = MailProperties.getInstance();
         propagateHostNames = Collections.emptySet();
+        allowFetchSingleHeaders = true;
+        allowFolderCaches = true;
     }
 
     @Override
@@ -191,6 +195,12 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
             logBuilder.append("\tIMAP allow folder caches: ").append(allowFolderCaches).append('\n');
         }
 
+        {
+            final String str = configuration.getProperty("com.openexchange.imap.allowFetchSingleHeaders", "true").trim();
+            allowFetchSingleHeaders = "true".equalsIgnoreCase(str);
+            logBuilder.append("\tIMAP allow FETCH single headers: ").append(allowFetchSingleHeaders).append('\n');
+        }
+
         {
             final String imapSortStr = configuration.getProperty("com.openexchange.imap.imapSort", "application").trim();
             imapSort = "imap".equalsIgnoreCase(imapSortStr);
@@ -201,7 +211,7 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
             final String imapSearchStr = configuration.getProperty("com.openexchange.imap.imapSearch", "imap").trim();
             forceImapSearch = "force-imap".equalsIgnoreCase(imapSearchStr);
             imapSearch = forceImapSearch || "imap".equalsIgnoreCase(imapSearchStr);
-            logBuilder.append("\tIMAP-Search: ").append(imapSearch).append(forceImapSearch ? " (forced)\n" : '\n');
+            logBuilder.append("\tIMAP-Search: ").append(imapSearch).append(forceImapSearch ? " (forced)\n" : "\n");
         }
 
         {
@@ -607,6 +617,11 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
         return allowFolderCaches;
     }
 
+    @Override
+    public boolean allowFetchSingleHeaders() {
+        return allowFetchSingleHeaders;
+    }
+
     /**
      * Gets the container type.
      *
