@@ -49,14 +49,22 @@
 
 package com.openexchange.imap.osgi;
 
+import static com.openexchange.java.Autoboxing.I;
+import java.io.Serializable;
+import java.util.LinkedHashSet;
+import java.util.List;
+import java.util.Set;
 import org.osgi.framework.BundleContext;
 import org.osgi.framework.ServiceReference;
 import org.osgi.util.tracker.ServiceTrackerCustomizer;
 import org.slf4j.Logger;
+import com.openexchange.caching.CacheKey;
 import com.openexchange.caching.events.CacheEvent;
 import com.openexchange.caching.events.CacheEventService;
 import com.openexchange.caching.events.CacheListener;
 import com.openexchange.imap.cache.ListLsubCache;
+import com.openexchange.java.util.Pair;
+import com.openexchange.mailaccount.Tools;
 
 
 /**
@@ -78,42 +86,50 @@ public final class ListLsubInvalidator implements CacheListener, ServiceTrackerC
      *
      * @param context The bundle context
      */
-    public ListLsubInvalidator(final BundleContext context) {
+    public ListLsubInvalidator(BundleContext context) {
         super();
         this.context = context;
     }
 
     @Override
-    public void onEvent(final Object sender, final CacheEvent cacheEvent, final boolean fromRemote) {
+    public void onEvent(Object sender, CacheEvent cacheEvent, boolean fromRemote) {
         if (fromRemote) {
             // Remotely received
             LOGGER.debug("Handling incoming remote cache event: {}", cacheEvent);
 
-            final String region = cacheEvent.getRegion();
-            if (REGION.equals(region)) {
-                final String key = cacheEvent.getKey().toString(); // <user-id> + "@" + <context-id>
-                final int pos = key.indexOf('@');
-                final int userId = Integer.parseInt(key.substring(0, pos));
-                final int contextId = Integer.parseInt(key.substring(pos + 1));
-                ListLsubCache.dropFor(userId, contextId);
+            if (REGION.equals(cacheEvent.getRegion())) {
+                List<Serializable> keys = cacheEvent.getKeys();
+                Set<Pair<Integer, Integer>> pairs = new LinkedHashSet<Pair<Integer,Integer>>(keys.size());
+                for (Serializable cacheKey : keys) {
+                    if (cacheKey instanceof CacheKey) {
+                        CacheKey ckey = (CacheKey) cacheKey;
+                        int contextId = Tools.getUnsignedInteger(cacheEvent.getGroupName());
+                        if (contextId > 0) {
+                            int userId = Integer.parseInt(ckey.getKeys()[0].toString());
+                            if (pairs.add(new Pair<Integer, Integer>(I(contextId), I(userId)))) {
+                                ListLsubCache.dropFor(userId, contextId, false);
+                            }
+                        }
+                    }
+                }
             }
         }
     }
 
     @Override
-    public CacheEventService addingService(final ServiceReference<CacheEventService> reference) {
-        final CacheEventService service = context.getService(reference);
+    public CacheEventService addingService(ServiceReference<CacheEventService> reference) {
+        CacheEventService service = context.getService(reference);
         service.addListener(REGION, this);
         return service;
     }
 
     @Override
-    public void modifiedService(final ServiceReference<CacheEventService> reference, final CacheEventService service) {
+    public void modifiedService(ServiceReference<CacheEventService> reference, CacheEventService service) {
         // Nothing to do
     }
 
     @Override
-    public void removedService(final ServiceReference<CacheEventService> reference, final CacheEventService service) {
+    public void removedService(ServiceReference<CacheEventService> reference, CacheEventService service) {
         service.removeListener(REGION, this);
         context.ungetService(reference);
     }
