@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2012 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2014 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -157,8 +157,7 @@ public final class IMAPCapabilityAndGreetingCache {
         }
         Future<CapabilityAndGreeting> f = map.get(address);
         if (null == f) {
-            final FutureTask<CapabilityAndGreeting> ft =
-                new FutureTask<CapabilityAndGreeting>(new CapabilityAndGreetingCallable(address, isSecure, imapProperties));
+            final FutureTask<CapabilityAndGreeting> ft = new FutureTask<CapabilityAndGreeting>(new CapabilityAndGreetingCallable(address, isSecure, imapProperties));
             f = map.putIfAbsent(address, ft);
             if (null == f) {
                 f = ft;
@@ -234,7 +233,7 @@ public final class IMAPCapabilityAndGreetingCache {
                 }
                 final InputStream in = s.getInputStream();
                 final OutputStream out = s.getOutputStream();
-                final com.openexchange.java.StringAllocator sb = new com.openexchange.java.StringAllocator(512);
+                final StringBuilder sb = new StringBuilder(512);
                 /*
                  * Read IMAP server greeting on connect
                  */
@@ -254,7 +253,7 @@ public final class IMAPCapabilityAndGreetingCache {
                     }
                 }
                 final String greeting = sb.toString();
-                sb.reinitTo(0);
+                sb.setLength(0);
                 if (skipLF) {
                     /*
                      * Consume final LF
@@ -305,7 +304,7 @@ public final class IMAPCapabilityAndGreetingCache {
                         }
                     } while (nextLine);
                     final String[] lines = SPLIT.split(sb.toString());
-                    sb.reinitTo(0);
+                    sb.setLength(0);
                     for (final String line : lines) {
                         if (!line.startsWith("A10 ")) {
                             sb.append(' ').append(line);
